<?php
namespace Datec\DatecFeusermanagement\Services;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Authentication services class
 *
 * PR: customized for login with remote interface of KIS
 */
class HashAuthenticationService extends \TYPO3\CMS\Sv\AuthenticationService {
	
	/**
	 * Initialize authentication service
	 * Saves remote authentication hash in session.
	 *
	 * @param string $mode Subtype of the service which is used to call the service.
	 * @param array $loginData Submitted login form data
	 * @param array $authInfo Information array. Holds submitted form data etc.
	 * @param AbstractUserAuthentication $pObj Parent object
	 * @return void
	 */
	public function initAuth($mode, &$loginData, $authInfo, $pObj) {
		parent::initAuth($mode, $loginData, $authInfo, $pObj);
		
		if (isset($_GET["tx_datecfeusermanagement"]["userAuthHash"])) {
			$hashParts = GeneralUtility::trimExplode('/', $_GET["tx_datecfeusermanagement"]["userAuthHash"]);
			$dbHash = $hashParts[0].'/'.md5($hashParts[1]);
			
			$user = $this->getUserByHash($dbHash);			
			if (!is_array($user)) {
				GeneralUtility::sysLog(sprintf('Login-attempt from %s (%s), for hash \'%s\', hash is wrong!', $this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $dbHash), 'Core', GeneralUtility::SYSLOG_SEVERITY_WARNING);
				return FALSE;
			}
			
			$loginData['uname'] = $user['username'];
			$loginData['uident'] = $user['password'];
			$loginData['userDBAuthHash'] = $dbHash;
			
			$this->login = $loginData;
		}
	}
	
	/**
	 * Find a user (eg. look up the user record in database when a login is sent)
	 *
	 * @return mixed User array or FALSE
	 */
	public function getUser() {
		if ($this->login['status'] !== 'login') {
			return FALSE;
		}
		if (!$this->login['userDBAuthHash']) {
			// Failed Login attempt (no authentification hash given)
			$this->writelog(255, 3, 3, 2, 'Login-attempt from %s (%s) for username \'%s\' with an empty authHash!', array(
					$this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $this->login['uname']
			));
			GeneralUtility::sysLog(sprintf('Login-attempt from %s (%s), for username \'%s\' with an empty authHash!', $this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $this->login['uname']), 'Core', GeneralUtility::SYSLOG_SEVERITY_WARNING);
			return FALSE;
		}
	
		$user = $this->getUserByHash($this->login['userDBAuthHash']);
		if (!is_array($user)) {
			// Failed login attempt (no username found)
			$this->writelog(255, 3, 3, 2, 'Login-attempt from %s (%s), username \'%s\' not found!!', array($this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $this->login['uname']));
			// Logout written to log
			GeneralUtility::sysLog(sprintf('Login-attempt from %s (%s), username \'%s\' not found!', $this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $this->login['uname']), 'core', GeneralUtility::SYSLOG_SEVERITY_WARNING);
		} else {
			if ($this->writeDevLog) {
				GeneralUtility::devLog('User found: ' . GeneralUtility::arrayToLogString($user, array($this->db_user['userid_column'], $this->db_user['username_column'])), AuthenticationService::class);
			}
		}
		return $user;
	}
	
	/**
	 * Authenticate a user (Check various conditions for the user that might invalidate its authentication, eg. password match, domain, IP, etc.)
	 *
	 * @param array $user Data of user.
	 * @return int >= 200: User authenticated successfully.
	 *                     No more checking is needed by other auth services.
	 *             >= 100: User not authenticated; this service is not responsible.
	 *                     Other auth services will be asked.
	 *             > 0:    User authenticated successfully.
	 *                     Other auth services will still be asked.
	 *             <= 0:   Authentication failed, no more checking needed
	 *                     by other auth services.
	 */
	public function authUser(array $user) {
		$OK = 100;
		if ($this->login['uname'] && isset($this->login['userDBAuthHash'])) {
			$dbHash = $this->login['userDBAuthHash'];
			$user = $this->getUserByHash($dbHash);			
			if (!is_array($user)) {
				// Failed login attempt (wrong userAuthHash) - write that to the log!
				if ($this->writeAttemptLog) {
					$this->writelog(255, 3, 3, 1, 'Login-attempt from %s (%s), username \'%s\', authHash not accepted!', array($this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $this->login['uname']));
					GeneralUtility::sysLog(sprintf('Login-attempt from %s (%s), username \'%s\', authHash not accepted!', $this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $this->login['uname']), 'core', GeneralUtility::SYSLOG_SEVERITY_WARNING);
				}
				if ($this->writeDevLog) {
					GeneralUtility::devLog('Password not accepted: ' . $this->login['uident'], RemoteAuthenticationService::class, 2);
				}
			} else {
				//$this->unsetUserHash($dbHash);
				$OK = 200;
			}
			// Checking the domain (lockToDomain)
			if ($user['lockToDomain'] && $user['lockToDomain'] !== $this->authInfo['HTTP_HOST']) {
				// Lock domain didn't match, so error:
				if ($this->writeAttemptLog) {
					$this->writelog(255, 3, 3, 1, 'Login-attempt from %s (%s), username \'%s\', locked domain \'%s\' did not match \'%s\'!', array($this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $user[$this->db_user['username_column']], $user['lockToDomain'], $this->authInfo['HTTP_HOST']));
					GeneralUtility::sysLog(sprintf('Login-attempt from %s (%s), username \'%s\', locked domain \'%s\' did not match \'%s\'!', $this->authInfo['REMOTE_ADDR'], $this->authInfo['REMOTE_HOST'], $user[$this->db_user['username_column']], $user['lockToDomain'], $this->authInfo['HTTP_HOST']), 'core', GeneralUtility::SYSLOG_SEVERITY_WARNING);
				}
				$OK = 0;
			}
		}
		return $OK;
	}
	
	/**
	 * Get the user identifiable by hash.
	 * 
	 * @param string $dbHash 
	 * @return array|boolean FALSE on error
	 */
	protected function getUserByHash($dbHash) {
		$where = ' tx_datec_feusermanagement_hashvalue="'.$dbHash.'"';
		$res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'fe_users', $where);
		if ($res) {
			$row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
		}
	
		return $row;
	}
	
	/**
	 * Deletes the hash from the database.
	 * 
	 * @param string $dbHash
	 * @return mixed FALSE on error
	 */
	protected function unsetUserHash($dbHash) {
		$where = ' tx_datec_feusermanagement_hashvalue="'.$dbHash.'"';
		$updateArr = array(
			'tx_datec_feusermanagement_hashvalue' => ''
		);
		$res = $GLOBALS['TYPO3_DB']->exec_UPDATEquery('fe_users', $where, $updateArr);
		
		return $res;
	}
	
}